import hashlib
import struct

from twisted.internet import defer, reactor
from twisted.web import client

import stratum.logger
log = stratum.logger.get_logger('proxy')

minimal_target = None
hashing_algo = None
ltc_scrypt = None

def set_hashing_algo(algo):
	global hashing_algo, minimal_target, ltc_scrypt
	if algo == 'doublesha':
		hashing_algo = 'doublesha'
		minimal_target = 0x00000000ffff0000000000000000000000000000000000000000000000000000
	else:
		if algo == 'scrypt':
			hashing_algo = 'scrypt'
			minimal_target = 0x0000ffff00000000000000000000000000000000000000000000000000000000
			
			try:
				import ltc_scrypt
			except:
				raise Exception("Unable to load ltc_scrypt module.")
		else:
			raise Exception("Unknown algo, only doublesha and scrypt supported.")
	

def show_message(msg):
    '''Repeatedly displays the message received from
    the server.'''
    log.warning("MESSAGE FROM THE SERVER OPERATOR: %s" % msg)
    log.warning("Restart proxy to discard the message")
    reactor.callLater(10, show_message, msg)

def format_hash(h):
    # For printing hashes to console
    return "%s" % h[:8]
  
def uint256_from_str(s):
    r = 0L
    t = struct.unpack("<IIIIIIII", s[:32])
    for i in xrange(8):
        r += t[i] << (i * 32)
    return r

def uint256_to_str(u):
    rs = ""
    for i in xrange(8):
        rs += struct.pack("<I", u & 0xFFFFFFFFL)
        u >>= 32
    return rs  

def reverse_hash(h):
    return struct.pack('>IIIIIIII', *struct.unpack('>IIIIIIII', h)[::-1])[::-1]
     
def doublesha(b):
    return hashlib.sha256(hashlib.sha256(b).digest()).digest()

def PoW(b):
	global hashing_algo, ltc_scrypt
	if hashing_algo == 'doublesha':
		return hashlib.sha256(hashlib.sha256(b).digest()).digest()
	else:
		return ltc_scrypt.getPoWHash(b)

def getMT():
	global minimal_target
	return minimal_target

def getAlgo():
	global hashing_algo
	return hashing_algo

@defer.inlineCallbacks
def detect_stratum(host, port):
    '''Perform getwork request to given
    host/port. If server respond, it will
    try to parse X-Stratum header.
    Not the most elegant code, but it works,
    because Stratum server should close the connection
    when client uses unknown payload.'''
        
    def get_raw_page(url, *args, **kwargs):
        scheme, host, port, path = client._parse(url)
        factory = client.HTTPClientFactory(url, *args, **kwargs)
        reactor.connectTCP(host, port, factory)
        return factory

    def _on_callback(_, d):d.callback(True)
    def _on_errback(_, d): d.callback(True)
    f = get_raw_page('http://%s:%d' % (host, port))
    
    d = defer.Deferred()
    f.deferred.addCallback(_on_callback, d)
    f.deferred.addErrback(_on_errback, d)
    (yield d)
    
    if not f.response_headers:
        # Most likely we're already connecting to Stratum
        defer.returnValue((host, port))
    
    header = f.response_headers.get('x-stratum', None)[0]
    if not header:
        # Looks like pool doesn't support stratum
        defer.returnValue(None) 
    
    if 'stratum+tcp://' not in header:
        # Invalid header or unsupported transport
        defer.returnValue(None)
    
    header = header.replace('stratum+tcp://', '').strip()
    host = header.split(':')    
    
    if len(host) == 1:
        # Port is not specified
        defer.returnValue((host[0], 3333))
    elif len(host) == 2:
        defer.returnValue((host[0], int(host[1])))
    
    defer.returnValue(None)